#!/bin/sh

if [ "x$1" = "xclean" ]; then
    echo Cleaning
    echo DONDOODODNODNODNODNODNO
    make distclean
    rm -f confi
    rm -rf autom4te.cache config.aux m4
    exit
fi

## Generate Makefile.am from MD5SUMS
PSOURCE=opuciplayer.cc
PNAME=`basename "${PSOURCE}" .cc`
echo "Generating Makefile.am from MD5SUMS"
# Update MD5SUMS
which ccbuild && (ccbuild md5 --recursive-include . "${PSOURCE}" > MD5SUMS)

SOURCES=`sed  -r 's/^[a-z0-9]+  //; s/ /\\ /' < MD5SUMS | tr '\n' ' '`

# Overwrite Makefile.am
cat > Makefile.am <<EOF
#THIS FILE IS AUTOMATICALLY GENERATED AUTOMATICALLY
PACKAGE = @PACKAGE@
VERSION = @VERSION@

AUTOMAKE_OPTIONS = subdir-objects

bin_PROGRAMS = ${PNAME}

#List of source files needed to build the executable:
${PNAME}_SOURCES = ${SOURCES}

INCLUDES = \$(DEPS_CFLAGS) -I. \
-I./ast \
-I./BatsException \
-I./Behavior \
-I./Behavior/BeamTo \
-I./Behavior/Empty \
-I./Behavior/GetFromSide \
-I./Behavior/GetUpFromBack \
-I./Behavior/GetUpFromBelly \
-I./Behavior/GoodPosition \
-I./Behavior/GoodPositionAngle \
-I./Behavior/GuardGoal \
-I./Behavior/KickTo \
-I./Behavior/MoveJointTo \
-I./Behavior/PrimitiveBehavior \
-I./Behavior/PrimitiveBehavior/Beam \
-I./Behavior/PrimitiveBehavior/MoveHingeJoint \
-I./Behavior/PrimitiveBehavior/MoveJoint \
-I./Behavior/PrimitiveBehavior/MoveUniversalJoint \
-I./Behavior/Score \
-I./Behavior/ScriptPlayer \
-I./Behavior/Sine \
-I./Behavior/Sine2 \
-I./Behavior/TurnTo \
-I./Behavior/Wait \
-I./Behavior/WalkStraightTo \
-I./Behavior/WalkTo \
-I./Behavior/canKickLeft \
-I./Behavior/canKickRight \
-I./Cerebellum \
-I./Conf \
-I./Distribution1 \
-I./Distribution1/NormalDistribution1 \
-I./Distribution3D \
-I./Distribution3D/MonteCarloDistribution \
-I./Distribution3D/NormalDistribution3D \
-I./FitnessFunction \
-I./FitnessFunction/FitnessTurn \
-I./FitnessFunction/FitnessWalk \
-I./Hashclasses \
-I./HumanoidAgent \
-I./HumanoidAgent/OpuciPlayer \
-I./Input \
-I./Logger \
-I./Localizer \
-I./Localizer/SimpleLocalizer \
-I./Math \
-I./Matrix4D \
-I./Matrix9 \
-I./Net \
-I./Parser \
-I./Path \
-I./PortableParser \
-I./Predicate \
-I./Preprocessor \
-I./Ref \
-I./RefAble \
-I./Singleton \
-I./SocketComm \
-I./Talent \
-I./Types \
-I./Vector3 \
-I./Vector4 \
-I./WorldModel \
-I./autom4te.cache \
-I./confinc \
-I./docs \
-I./docs/TDP \
-I./docs/agent_paper \
-I./docs/howtorobocop \
-I./docs/img \
-I./global \
-I./util \
-I./Behavior/Dribble \
-I./Behavior/FallBackward \
-I./Behavior/FallForward \
-I./Behavior/FallLeftSide \
-I./Behavior/FallRightSide \
-I./Behavior/Freeze \
-I./Behavior/GetUp \
-I./Behavior/StandBy \
-I./Behavior/WaitEnemy \
-I./Behavior/KeepingGoal \
-I./Behavior/Guard \
-I./Behavior/Intercept \
-I./Behavior/DribbleForward \
-I./Behavior/DribbleSide \
-I./Behavior/TurnForDribble \
-I./Behavior/KickPosition \
-I./Behavior/KickPositionDefender \
-I./Behavior/BestPosition \
-I./Behavior/BestAngle \
-I./Behavior/TurnAndWait \
-I./Behavior/TurnRightStrict \
-I./Behavior/TurnLeftStrict \
-I./Behavior/Search \
-I./Behavior/GoalKick \
-I./Behavior/Block \
-I./Behavior/KeeperWalk \
-I./Behavior/KeeperGetUp \
-I./Behavior/KeeperGoodPosition \
-I./Behavior/KeeperKickPosition \
-I./Behavior/ReadyToGuard \
-I./Behavior/RightStep \
-I./Behavior/LeftStep \
-I./Behavior/SupportDefense \
-I./Behavior/DirectDefense \
-I./Behavior/Defense \
-I./Behavior/WaitAndSee \
-I./Behavior/Stop \
-I./Behavior/KickRightR \
-I./Behavior/KickLeftL \
-I./Behavior/KickStraight \
-I./Behavior/Stay \
-I./Behavior/AvoidOthers \
-I./Behavior/DiagKickLeft \
-I./Behavior/DiagKickRight \
-I./Behavior/DiagPass \
-I./Behavior/FaceKickLeft \
-I./Behavior/FaceKickRight \
-I./Behavior/FacePass \
-I./Behavior/GoToBall \
-I./Behavior/AssaultGoal \
-I./Behavior/Pass \
-I./Behavior/Position \
-I./Behavior/ReturnToPosition \
-I./Behavior/SideKickLeft \
-I./Behavior/SideKickRight \
-I./Behavior/SidePass \
-I./Behavior/WalkAndTurn \
-I./Behavior/CornerKick \
-I./Behavior/CornerKickPosition \
-I./Behavior/CornerKickRight \
-I./Behavior/CornerKickLeft \
-I./GamepadInput \
-I./Behavior/GamepadPlay \
-I./Behavior/GamepadWalk \
-I./Behavior/GamepadOmnidirectionalWalk \
-I./Behavior/GamepadWalkBackward \
-I./Behavior/GamepadMiniWalkForward \
-I./Behavior/GamepadMiniWalkBackward \
-I./Behavior/GamepadStrafeRight \
-I./Behavior/GamepadStrafeLeft \
-I./Behavior/GamepadGetUp \
-I./Behavior/GamepadKickRight \
-I./Behavior/GamepadKickLeft \
-I./Behavior/GamepadTurnRight \
-I./Behavior/GamepadTurnLeft \
-I./Behavior/GamepadFallForward \
-I./Behavior/GamepadFallBackward \
-I./Behavior/GamepadFallRightside \
-I./Behavior/GamepadFallLeftside \
-I./Behavior/GamepadBeam \
-I./Behavior/GamepadJoy \
-I./Behavior/GamepadStepping \
-I./VD3Connector

${PNAME}_LDADD = \$(DEPS_LIBS)
EOF

rm -f config.cache acconfig.h

echo "- aclocal." && \
aclocal && \
echo "- autoconf." && \
autoconf && \
echo "- automake." && \
automake --add-missing --copy

